/* Find.c -- Finding things in your world
 *
 *   These routines all start with the word find, then contain three words
 * organized to tell you all you need to know about the routine, the format
 * is as follows:
 *
 *    find_<what>_<based on>_<from where>
 *
 * For example, the routine find_door_dir_room will find a door in a given
 * room, based on the direction of the door that you want.
 *
 * The standard find routines will return pointers, ifind routines will
 * return indexes you can use to index arrays and whatnot...
 ***/

#include <stdio.h>
#include <string.h>
#include "ymw.h"
#include "find.h"

DOOR *find_door_dir_room( ROOM *room, int dir )
{
    int j;

    for( j = 0; j < room->num_doors; j++ ){
       if( room->door[j].dir == dir )
         return &room->door[j];
       }

    return NULL;
}

int ifind_door_dir_room( ROOM *room, int dir )
{
    int j;

    for( j = 0; j < room->num_doors; j++ ){
       if( room->door[j].dir == dir )
         return j;
       }

    return -1;
}

DOOR *find_door_vnum_world( WORLD *world, int zone, int room, int dir )
{
    int j, k;

    for( j = 0; j < world->num_zones; j++ ){
       if( world->zone[j].vnum == zone ){
         for( k = 0; k < world->zone[j].num_rooms; k++ ){
            if( world->zone[j].room[k].vnum == room ){
              return find_door_dir_room( &world->zone[j].room[k], dir );
              }
            }
         }
       }

    return NULL;
}

DOOR *find_door_croom_room( ROOM *room, int czone, int croom )
{
    int j;

    for( j = 0; j < room->num_doors; j++ ){
       if( room->door[j].czone == czone &&
           room->door[j].croom == croom ){
         return &room->door[j];
         }
       }

    return NULL;
}

MOBILE *find_mobile_vnum_zone( ZONE *zone, int vnum )
{
   int j;

   for( j = 0; j < zone->num_mobiles; j++ ){
      if( zone->mobile[j].vnum == vnum ){
        return &zone->mobile[j];
        }
      }

   return NULL;
}

int ifind_mobile_vnum_zone( ZONE *zone, int vnum )
{
   int j;

   for( j = 0; j < zone->num_mobiles; j++ ){
      if( zone->mobile[j].vnum == vnum ){
        return j;
        }
      }

   return -1;
}

OBJECT *find_object_vnum_zone( ZONE *zone, int vnum )
{
    int j;

    for( j = 0; j < zone->num_objects; j++ ){
       if( zone->object[j].vnum == vnum ){
         return &zone->object[j];
         }
       }

    return NULL;
}

int ifind_object_vnum_zone( ZONE *zone, int vnum )
{
    int j;

    for( j = 0; j < zone->num_objects; j++ ){
       if( zone->object[j].vnum == vnum ){
         return j;
         }
       }

    return -1;
}

OBJECT *find_object_short_desc_zone( ZONE *zone, char *short_desc )
{
    int j;

    for( j = 0; j < zone->num_objects; j++ ){
       if( strcmp( zone->object[j].short_desc, short_desc ) == 0 ){
         return &zone->object[j];
         }
       }

    return NULL;
}

int ifind_object_short_desc_zone( ZONE *zone, char *short_desc )
{
    int j;

    for( j = 0; j < zone->num_objects; j++ ){
       if( strcmp( zone->object[j].short_desc, short_desc ) == 0 ){
         return j;
         }
       }

    return -1;
}

ROOM *find_room_vnum_zone( ZONE *zone, int vnum )
{
    int j;

    for( j = 0; j < zone->num_rooms; j++ ){
       if( zone->room[j].vnum == vnum ){
         return &zone->room[j];
         }
       }

    return NULL;
}

int ifind_room_vnum_zone( ZONE *zone, int vnum )
{
    int j;

    for( j = 0; j < zone->num_rooms; j++ ){
       if( zone->room[j].vnum == vnum ){
         return j;
         }
       }

    return -1;
}

ROOM *find_room_vnum_world( WORLD *world, int zvnum, int rvnum )
{
    int j;

    for( j = 0; j < world->num_zones; j++ ){
       if( world->zone[j].vnum == zvnum ){
         return find_room_vnum_zone( &world->zone[j], rvnum );
         }
       }

    return NULL;
}

int ifind_room_vnum_world( WORLD *world, int zvnum, int rvnum )
{
    int j;

    for( j = 0; j < world->num_zones; j++ ){
       if( world->zone[j].vnum == zvnum ){
         return ifind_room_vnum_zone( &world->zone[j], rvnum );
         }
       }

    return -1;
}

ZONE *find_zone_vnum_world( WORLD *world, int vnum )
{
    int j;

    for( j = 0; j < world->num_zones; j++ ){
       if( world->zone[j].vnum == vnum ){
         return &world->zone[j];
         }
       }

    return NULL;
}

int ifind_zone_vnum_world( WORLD *world, int vnum )
{
    int j;

    for( j = 0; j < world->num_zones; j++ ){
       if( world->zone[j].vnum == vnum ){
         return j;
         }
       }

    return -1;
}

int ifind_bvitem( BVLIST *list, int bv )
{
    int j;

    for( j = 0; list[j].desc; j++ ){
       if( list[j].bitpos == bv )
         return j;
       }

    return -1;
}

MSEL_ITEM *find_msel_item( MSEL *msel, int hi_ind, int lo_ind )
{
    MSEL_ITEM *cur;

    if( (cur = msel->base) == NULL )
      return NULL;

    do{
      if( cur->next->hi_ind == hi_ind &&
          cur->next->lo_ind == lo_ind ){
        return cur;
        }
      cur = cur->next;
      } while( cur != msel->base );

    return NULL;
}
